#include <stdio.h>
#include <fcntl.h>
#include <strings.h>
#include <db.h>

DB *open_db __P((char *, recno_t *));
int write_random_tagline __P((DB *, recno_t));

main()
{
	int err;
	DB *db;
	recno_t size;		/* # taglines in database */

	srandom(time(NULL));	/* Seed the random number generator */

	if ((db = open_db(DB_FILE, &size)) == NULL)
		exit(1);
	write_random_tagline(db, size);
	/* Clean up and exit */
	db->close(db);
}

/* open_db
 * Open the database 'dbname' for reading, and return a pointer to
 * the DB structure.
 * Sets 'nrecs' to the number of records in the database.
 */
DB *open_db(dbname, nrecs)
char *dbname;
recno_t *nrecs;
{
	DB *retval;
	DBT key, item;
	recno_t recnum;
	int err;

	/* Open the tagline database */
	if ((retval = dbopen(dbname, O_RDONLY, 0444, DB_RECNO, NULL)) == NULL)
	{
		fprintf(stderr, "open_db: Can't open database.\n");
		perror("dbopen");
		return(NULL);
	}

	/* Get the number of records in the database */
	/* Initialize 'key' and 'item' */
	key.data = &recnum;
	key.size = sizeof(recno_t);

	if ((err = retval->seq(retval, &key, &item, R_LAST)) < 0)
	{
		fprintf(stderr, "open_db: Can't seek to end.\n");
		perror("db->seq");
		retval->close(retval);
		return(NULL);
	}
	*nrecs = *((recno_t *) key.data);

	return(retval);
}

/* write_random_tagline
 * Pick a random tagline from the DB database 'db', and write it
 * to 'sig'.
 * 'nrecs' gives the number of records in the database.
 * All instances of \r in the tagline will be converted to \n before
 * printing.
 */
int write_random_tagline(db,nrecords)
DB *db;
recno_t nrecords;
{
	DBT key, item;
	recno_t keynum;
	int err;
	char *ret;

	key.data = &keynum;
	key.size = sizeof(keynum);

	/* Pathological case: empty tagline database */
	if (nrecords <= 0)
		return(0);

	/* Pick a random tagline */
	keynum = (random() % nrecords)+1;

	if ((err = db->seq(db, &key, &item, R_CURSOR)) < 0)
	{
		fprintf(stderr, "write_random_tagline: Can't seek.\n");
		perror("db->seq");
		return(err);
	}
	/* Terminate the string, just to make sure */
	((char *) item.data)[item.size] = '\0';

	/* Replace \r with \n throughout */
	ret = item.data;
	while ((ret = index(ret, '\r')) != NULL)
		*ret++ = '\n';

	/* And finally print the tagline */
	printf("%s\n", (char *) item.data);

	return(0);
}
